"""
Example 1
PERT 
file name: Example1.py
Copyright Log Opt Co., Ltd.

Consider a 5-activity problem with precedence constraints between the activities.
Such a problem is called PERT (Program Evaluation and Review Technique).
The processing times (durations) of the activities are kept in the dictionary
 duration ={1:13, 2:25, 3:15, 4:27, 5:22 }.
Precedence constraints are given by:
 Activity 1 -> Activity 3; Activity 2 -> Activity 4;
 Activity 3 -> Activity 4; and Activity 3 -> Activity 5.
The objective is to find the maximum completion time (makespan) for all 5 activities.
"""

from optseq import *
m1=Model()
duration ={1:13, 2:25, 3:15, 4:27, 5:22 }
act={}
mode={}        
for i in duration: 
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    act[i].addModes(mode[i])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[3])
m1.addTemporal(act[2],act[4])
m1.addTemporal(act[2],act[5])
m1.addTemporal(act[3],act[4])

print (m1)
m1.Params.TimeLimit=1
m1.Params.Makespan=True
m1.optimize()
m1.write("chart1.txt")

##output: 
##activity Act[1]
##mode duration 13
##activity Act[2]
##mode duration 25
##activity Act[3]
##mode duration 15
##activity Act[4]
##mode duration 27
##activity Act[5]
##mode duration 22
##
##temporal Act[1] Act[3] type CS delay 0
##temporal Act[2] Act[4] type CS delay 0
##temporal Act[2] Act[5] type CS delay 0
##temporal Act[3] Act[4] type CS delay 0
##
##activity sink duedate 0 
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 7

##objective value = 55 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 55/55

##--- best activity list ---

##source Act[2] Act[5] Act[1] Act[3] Act[4] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 55 55

##Act[1],---, 0 0--13 13

##Act[2],---, 0 0--25 25

##Act[3],---, 13 13--28 28

##Act[4],---, 28 28--55 55

##Act[5],---, 25 25--47 47

##--- tardy activity ---

##sink: 55

##--- resource residuals ---

##

##objective value = 55

##cpu time = 0.00/1.00(s)

##iteration = 1/14299

##
##
##source --- 0 0
##sink --- 55 55
##Act[1] --- 0 13
##Act[2] --- 0 25
##Act[3] --- 13 28
##Act[4] --- 28 55
##Act[5] --- 25 47
##planning horizon= 55

#chart1.txt
##activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31|32|33|34|35|36|37|38|39|40|41|42|43|44|45|46|47|48|49|50|51|52|53|54|55|
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
##  Act[1]   Mode[1]      13    |==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      25    |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[3]   Mode[3]      15    |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[4]   Mode[4]      27    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|
##  Act[5]   Mode[5]      22    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
##   resource usage/capacity    | 
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


