"""
Example 5
Resource Constrained Scheduling
file name: Example5.py
Copyright Log Opt Co., Ltd.

Consider a 4-activity problem with a resource whose capacity is 2 units on day 1, 2, 4, 5, 6, and 1 unit on day 3.
The processing times (durations) of the activities are kept in the dictionary
 duration ={1:1,2:3,3:2,4:2}.
Precedence constraints are give by:
 Activity 1 -> Activity 2;
 Activity 1 -> Activity 3;
 Activity 2 -> Activity 4.
Activity 1 requires 2 units of resource the first day.
Activity 2 requires 2 units of resource the first day and 1 unit on other days.
Activity 3 requires 1 unit of resource all the days.
Activity 4 requires 1 unit of the resource the first day and 2 units on the second day.
The objective is to find the maximum completion time (makespan).
"""

from optseq import *

m1=Model()                
duration ={1:1,2:3,3:2,4:2}
req={}
req[1]={(0,1):2 }
req[2]={(0,1):2 ,(1,3):1}
req[3]={(0,2):1 }
req[4]={(0,1):1,(1,2):2 }

res=m1.addResource("worker")
res.addCapacity(0,2,2)
res.addCapacity(2,3,1)
res.addCapacity(3,"inf",2)

act={}
mode={}

for i in duration: 
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,requirement=req[i])
    act[i].addModes(mode[i])                                            

#temporal (precedense) constraints
m1.addTemporal(act[1],act[2])
m1.addTemporal(act[1],act[3])
m1.addTemporal(act[2],act[4])

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart5.txt")

##output: 
##resource worker 
## interval 0 2 capacity 2
## interval 2 3 capacity 1
## interval 3 inf capacity 2
##activity Act[1]
##mode duration 1
## worker interval 0 1 requirement 2
##activity Act[2]
##mode duration 3
## worker interval 0 1 requirement 2
##interval 1 3 requirement 1
##activity Act[3]
##mode duration 2
## worker interval 0 2 requirement 1
##activity Act[4]
##mode duration 2
## worker interval 0 1 requirement 1
##interval 1 2 requirement 2
##
##temporal Act[1] Act[2] type CS delay 0
##temporal Act[1] Act[3] type CS delay 0
##temporal Act[2] Act[4] type CS delay 0
##
##activity sink duedate 0 
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 6

##objective value = 6 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 6/6

##--- best activity list ---

##source Act[1] Act[2] Act[3] Act[4] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 6 6

##Act[1],---, 0 0--1 1

##Act[2],---, 1 1--4 4

##Act[3],---, 3 3--5 5

##Act[4],---, 4 4--6 6

##--- tardy activity ---

##sink: 6

##--- resource residuals ---

##worker: [0,6] 0 

##

##objective value = 6

##cpu time = 0.00/1.00(s)

##iteration = 1/10916

##
##
##source --- 0 0
##sink --- 6 6
##Act[1] --- 0 1
##Act[2] --- 1 4
##Act[3] --- 3 5
##Act[4] --- 4 6
##planning horizon= 6

#chart5.txt
##  activity    mode   duration |1|2|3|4|5|6|
##------------------------------------------|
##  Act[1]   Mode[1]      1     |=| | | | | |
##  Act[2]   Mode[2]      3     | |=|=|=| | |
##  Act[3]   Mode[3]      2     | | | |=|=| |
##  Act[4]   Mode[4]      2     | | | | |=|=|
##------------------------------------------
##   resource usage/capacity    | 
##------------------------------------------
##            worker            |2|2|1|2|2|2|
##                              |2|2|1|2|2|2|
##------------------------------------------



