"""
Example 6
Minimizing the Tardiness
file name: Example6.py
Copyright Log Opt Co., Ltd.

Consider a 4-activity problem with one machine resource.
The due dates and processing times (durations) of the activities are kept in the dictionaries
 due={1:5,2:9,3:6,4:4} and
 duration={1:1, 2:2, 3:3, 4:4 }, respectively.
We have to pay tardiness penalty by the amount the completion time of the activity exceeds its due date.
The objective is to find the schedule that minimizes total tardiness penalty cost.
"""
from optseq import *

m1=Model()                
due={1:5,2:9,3:6,4:4}
duration={1:1, 2:2, 3:3, 4:4 }

res=m1.addResource("writer")
res.addCapacity(0,"inf",1)

act={}
mode={}

for i in duration: 
    act[i]=m1.addActivity("Act[{0}]".format(i),duedate=due[i])
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,{(0,"inf"):1})
    act[i].addModes(mode[i])                                            

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=False
m1.optimize()
m1.write("chart6.txt")

##output: 
##resource writer 
## interval 0 inf capacity 1
##activity Act[1]
## duedate 5
## weight 1
##mode duration 1
## writer interval 0 inf requirement 1
##activity Act[2]
## duedate 9
## weight 1
##mode duration 2
## writer interval 0 inf requirement 1
##activity Act[3]
## duedate 6
## weight 1
##mode duration 3
## writer interval 0 inf requirement 1
##activity Act[4]
## duedate 4
## weight 1
##mode duration 4
## writer interval 0 inf requirement 1
##
##
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 6

##objective value = 3 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 3/3

##

##--- best activity list ---

##source Act[4] Act[1] Act[3] Act[2] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 10 10

##Act[1],---, 4 4--5 5

##Act[2],---, 8 8--10 10

##Act[3],---, 5 5--8 8

##Act[4],---, 0 0--4 4

##--- tardy activity ---

##Act[2]: 1

##Act[3]: 2

##--- resource residuals ---

##writer: [0,10] 0 

##

##objective value = 3

##cpu time = 0.00/1.00(s)

##iteration = 0/15809

##
##
##source --- 0 0
##sink --- 10 10
##Act[1] --- 4 5
##Act[2] --- 8 10
##Act[3] --- 5 8
##Act[4] --- 0 4
##planning horizon= 10

#chart6.txt
##  activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|
##------------------------------------------------------------|
##  Act[1]   Mode[1]      1     |  |  |  |  |==|  |  |  |  |  |
##  Act[2]   Mode[2]      2     |  |  |  |  |  |  |  |  |==|==|
##  Act[3]   Mode[3]      3     |  |  |  |  |  |==|==|==|  |  |
##  Act[4]   Mode[4]      4     |==|==|==|==|  |  |  |  |  |  |
##------------------------------------------------------------
##   resource usage/capacity    | 
##------------------------------------------------------------
##            writer            | 1| 1| 1| 1| 1| 1| 1| 1| 1| 1|
##                              | 1| 1| 1| 1| 1| 1| 1| 1| 1| 1|

