"""
Example 7
CPM(critical path method) -- nonrenewable resource 
file name: Example7.py
Copyright Log Opt Co., Ltd.

Consider the same scenario as in Example 1. 
Here, activities have a standard mode and an express mode whose durations are 
durationA = {1:13, 2:25, 3:15, 4:27, 5:22 } and
durationB = {1:10, 2:20, 3:10, 4:25, 5:20 }, respectively.
Express modes require additional costs; they can be processed at most 4.
Find the makespan under the restriction. 

"""

from optseq import *

m1=Model()
Jobs=[1,2,3,4,5]
durationA = {1:13, 2:25, 3:15, 4:27, 5:22 }
durationB = {1:10, 2:20, 3:10, 4:25, 5:20 }

act={}
mode={}
for i in Jobs:
    mode[i,1]=Mode("Mode[{0}][1]".format(i),durationA[i])
    mode[i,2]=Mode("Mode[{0}][2]".format(i),durationB[i])
    act[i]=m1.addActivity("Act[{0}] ".format(i))
    act[i].addModes(mode[i,1],mode[i,2])


res=m1.addResource("money",rhs=4,direction="<=")

for i in Jobs:
    res.addTerms(1,act[i],mode[i,2])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[3])
m1.addTemporal(act[2],act[4])
m1.addTemporal(act[2],act[5])
m1.addTemporal(act[3],act[4])

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart7.txt")

##output: 
##mode Mode[2][1] duration 25
##mode Mode[3][1] duration 15
##mode Mode[1][1] duration 13
##mode Mode[3][2] duration 10
##mode Mode[1][2] duration 10
##mode Mode[5][2] duration 20
##mode Mode[4][2] duration 25
##mode Mode[4][1] duration 27
##mode Mode[2][2] duration 20
##mode Mode[5][1] duration 22
##activity Act[1]
##Mode[1][1] Mode[1][2] 
##activity Act[2]
##Mode[2][1] Mode[2][2] 
##activity Act[3]
##Mode[3][1] Mode[3][2] 
##activity Act[4]
##Mode[4][1] Mode[4][2] 
##activity Act[5]
##Mode[5][1] Mode[5][2] 
##
##temporal Act[1] Act[3] type CS delay 0
##temporal Act[2] Act[4] type CS delay 0
##temporal Act[2] Act[5] type CS delay 0
##temporal Act[3] Act[4] type CS delay 0
##nonrenewable 1(Act[1],Mode[1][2]) 1(Act[2],Mode[2][2]) 1(Act[3],Mode[3][2]) 1(Act[4],Mode[4][2]) 1(Act[5],Mode[5][2]) <=4
##activity sink duedate 0 
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 7

##objective value = 45 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 45/45

##

##--- best activity list ---

##source Act[2] Act[5] Act[1] Act[3] Act[4] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 45 45

##Act[1],Mode[1][2], 0 0--10 10

##Act[2],Mode[2][2], 0 0--20 20

##Act[3],Mode[3][2], 10 10--20 20

##Act[4],Mode[4][2], 20 20--45 45

##Act[5],Mode[5][1], 20 20--42 42

##--- tardy activity ---

##sink: 45

##--- resource residuals ---

##

##objective value = 45

##cpu time = 0.00/1.00(s)

##iteration = 0/29478

##
##
##source --- 0 0
##sink --- 45 45
##Act[1] Mode[1][2] 0 10
##Act[2] Mode[2][2] 0 20

#chart7.txt
##activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31|32|33|34|35|36|37|38|39|40|41|42|43|44|45|
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------|
##  Act[1]  Mode[1][2]    10    |==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]  Mode[2][2]    20    |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[3]  Mode[3][2]    10    |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[4]  Mode[4][2]    25    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|
##  Act[5]  Mode[5][1]    22    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------
##   resource usage/capacity    | 
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------

