"""
Example 8
Temporal Constraints
file name: Example8.py
Copyright Log Opt Co., Ltd.

Consider the same scenario as in Example 1. 
Here, we add an additional temporal constraint; Activity 3 and Activity 5 must be start simultaneously.
Find the makespan under this new condition. 

"""

from optseq import *


m1=Model()                
durationA = {1:13, 2:25, 3:15, 4:27, 5:22 }

act={}
mode={}

for i in durationA:
    act[i]=m1.addActivity("Act[{0}]".format(i))
    mode[i]=Mode("Mode[{0}]".format(i),durationA[i])
    act[i].addModes(mode[i])

#temporal (precedense) constraints
m1.addTemporal(act[1],act[3])
m1.addTemporal(act[2],act[4])
m1.addTemporal(act[2],act[5])
m1.addTemporal(act[3],act[4])

#act[3] and act[5] start at the same time
m1.addTemporal(act[3],act[5],"SS",0)
m1.addTemporal(act[5],act[3],"SS",0)

# act[5] begins at time 50 
#m1.addTemporal("source",act[5],"SS",delay=50)
#m1.addTemporal(act[5],"source","SS",delay=-50)

m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=True
m1.optimize()
m1.write("chart8.txt")

##output: 
##activity Act[1]
##mode duration 13
##activity Act[2]
##mode duration 25
##activity Act[3]
##mode duration 15
##activity Act[4]
##mode duration 27
##activity Act[5]
##mode duration 22
##
##temporal Act[1] Act[3] type CS delay 0
##temporal Act[2] Act[4] type CS delay 0
##temporal Act[2] Act[5] type CS delay 0
##temporal Act[3] Act[4] type CS delay 0
##temporal Act[3] Act[5] type SS delay 0
##temporal Act[5] Act[3] type SS delay 0
##
##activity sink duedate 0 
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 6

##objective value = 67 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 67/67

##

##--- best activity list ---

##source Act[2] Act[1] Act[3] Act[5] Act[4] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 67 67

##Act[1],---, 0 0--13 13

##Act[2],---, 0 0--25 25

##Act[3],---, 25 25--40 40

##Act[4],---, 40 40--67 67

##Act[5],---, 25 25--47 47

##--- tardy activity ---

##sink: 67

##--- resource residuals ---

##

##objective value = 67

##cpu time = 0.00/1.00(s)

##iteration = 0/21037

##
##
##source --- 0 0
##sink --- 67 67
##Act[1] --- 0 13
##Act[2] --- 0 25
##Act[3] --- 25 40
##Act[4] --- 40 67
##Act[5] --- 25 47
##planning horizon= 67

#chart8.txt
##activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|25|26|27|28|29|30|31|32|33|34|35|36|37|38|39|40|41|42|43|44|45|46|47|48|49|50|51|52|53|54|55|56|57|58|59|60|61|62|63|64|65|66|67|
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
##  Act[1]   Mode[1]      13    |==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      25    |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[3]   Mode[3]      15    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##  Act[4]   Mode[4]      27    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|
##  Act[5]   Mode[5]      22    |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|==|  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
##   resource usage/capacity    | 
##---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

