"""
Example 9
Breakable Activity 
file name: Example9.py
Copyright Log Opt Co., Ltd.

Consider the same scenario as in Example 6. 
Here, Activities 2 and 3 can interrupt their processing and re-start after one unit of break time.
Find the optimal schedule (minimum tardiness solution) under this new condition.
"""

from optseq import *

m1=Model()                

due={1:5,2:9,3:6,4:4}
duration={1:1, 2:2, 3:3, 4:4 }

res=m1.addResource("writer")
res.addCapacity(0,3,1)
res.addCapacity(4,6,1)
res.addCapacity(7,10,1)
res.addCapacity(11,"inf",1)

act={}
mode={}

for i in duration: 
    act[i]=m1.addActivity("Act[{0}]".format(i),duedate=due[i])
    mode[i]=Mode("Mode[{0}]".format(i),duration[i])
    mode[i].addResource(res,{(0,"inf"):1})
    #mode[i].addBreak(0,"inf",1)
    mode[i].addBreak(0,duration[i],1)
    #mode[i].addResource(res,{(0,"inf"):1},"break")       
    act[i].addModes(mode[i])
    
print (m1)
#m1.Params.Initial="Example9initial.txt"
m1.Params.TimeLimit=1
m1.Params.OutputFlag=True
m1.Params.Makespan=False
m1.optimize()
m1.write("chart9.txt")


##output: 
##resource writer 
## interval 0 3 capacity 1
## interval 4 6 capacity 1
## interval 7 10 capacity 1
## interval 11 inf capacity 1
##activity Act[1]
## duedate 5
## weight 1
##mode duration 1
## writer interval 0 inf requirement 1
## break interval 0 inf max 1
##activity Act[2]
## duedate 9
## weight 1
##mode duration 2
## writer interval 0 inf requirement 1
## break interval 0 inf max 1
##activity Act[3]
## duedate 6
## weight 1
##mode duration 3
## writer interval 0 inf requirement 1
## break interval 0 inf max 1
##activity Act[4]
## duedate 4
## weight 1
##mode duration 4
## writer interval 0 inf requirement 1
## break interval 0 inf max 1
##
##
##
### reading data ... done: 0.00(s)

### random seed: 1

### tabu tenure: 0

### cpu time limit: 1.00(s)

### iteration limit: 1073741823

### computing all-pairs longest paths and strongly connected components ... done

###scc 6

##objective value = 10 (cpu time = 0.00(s), iteration = 0)

##0: 0.00(s): 10/10

##objective value = 9 (cpu time = 0.00(s), iteration = 1)

##

##--- best activity list ---

##source Act[4] Act[1] Act[2] Act[3] sink 

##

##--- best solution ---

##source,---, 0 0

##sink,---, 13 13

##Act[1],---, 4 5--6 6

##Act[2],---, 6 7--9 9

##Act[3],---, 8 9--10 11--13 13

##Act[4],---, 0 0--3 4--5 5

##--- tardy activity ---

##Act[1]: 1

##Act[3]: 7

##Act[4]: 1

##--- resource residuals ---

##writer: [0,13] 0 

##

##objective value = 9

##cpu time = 0.00/1.00(s)

##iteration = 1/10868

##
##
##source --- 0 0
##sink --- 13 13
##Act[1] --- 4 6
##Act[2] --- 6 9
##Act[3] --- 8 13
##Act[4] --- 0 5
##planning horizon= 13

#chart9.txt
##  activity    mode   duration | 1| 2| 3| 4| 5| 6| 7| 8| 9|10|11|12|13|
##---------------------------------------------------------------------|
##  Act[1]   Mode[1]      1     |  |  |  |  |..|==|  |  |  |  |  |  |  |
##  Act[2]   Mode[2]      2     |  |  |  |  |  |  |..|==|==|  |  |  |  |
##  Act[3]   Mode[3]      3     |  |  |  |  |  |  |  |  |..|==|..|==|==|
##  Act[4]   Mode[4]      4     |==|==|==|..|==|  |  |  |  |  |  |  |  |
##---------------------------------------------------------------------
##   resource usage/capacity    | 
##---------------------------------------------------------------------
##            writer            | 1| 1| 1| 0| 1| 1| 0| 1| 1| 1| 0| 1| 1|
##                              | 1| 1| 1| 0| 1| 1| 0| 1| 1| 1| 0| 1| 1|
##---------------------------------------------------------------------
